@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # (c) Copyright by Pierre-Henri Wuillemin, UPMC, 2017
 # (pierre-henri.wuillemin@lip6.fr)
 # Permission to use, copy, modify, and distribute this
@@ -21,7 +20,6 @@
 """
 tools for BN in jupyter notebook
 """
-from __future__ import print_function
 
 import time
 import re
@@ -40,16 +38,14 @@ import matplotlib as mpl
 import matplotlib.pyplot as plt
 
 try:
-  from matplotlib_inline.backend_inline import set_matplotlib_formats as set_matplotlib_formats
-except ImportError: # because of python 2.7, matplotlib-inline cannot be part of requirements.txt
-  def set_matplotlib_formats(*args,**kwargs):
+  from matplotlib_inline.backend_inline import set_matplotlib_formats
+except ImportError:  # because of python 2.7, matplotlib-inline cannot be part of requirements.txt
+  def set_matplotlib_formats(*args, **kwargs):
     # dummy version when no matplotlib_inline package
-    if sys.version[0]=="3":
-      print("** pyAgrum** For better visualizations, please install matplotlib-inline.")
-    pass
+    print("** pyAgrum** For better visualizations, please install matplotlib-inline.")
 
 import numpy as np
-import pydotplus as dot
+import pydot as dot
 
 import IPython.core.display
 import IPython.core.pylabtools
@@ -63,6 +59,11 @@ from pyAgrum.lib.mn2graph import MN2UGdot, MNinference2UGdot
 from pyAgrum.lib.mn2graph import MN2FactorGraphdot, MNinference2FactorGraphdot
 from pyAgrum.lib.bn_vs_bn import GraphicalBNComparator
 from pyAgrum.lib.proba_histogram import proba2histo, probaMinMaxH
+from pyAgrum.lib.image import prepareShowInference,prepareLinksForSVG
+
+from pyAgrum.lib._colors import setDarkTheme,setLightTheme,getBlackInTheme
+
+import pyAgrum.lib._colors as gumcols
 
 
 class FlowLayout(object):
@@ -92,42 +93,66 @@ class FlowLayout(object):
       }
       </style>
       """
+    return self
 
-  def _getTitle(self, title):
-    if title == "":
+  def _getCaption(self, caption):
+    if caption == "":
       return ""
-    return f"<br><center><small><em>{title}</em></small></center>"
+    return f"<br><center><small><em>{caption}</em></small></center>"
 
-  def add_html(self, html, title=""):
+  def add_html(self, html, caption=None, title=None):
     """
-    add an html element in the row
+    add an html element in the row (title is an obsolete parameter)
     """
-    self.sHtml += f'<div class="floating-box">{html}{self._getTitle(title)}</div>'
+    if caption is None:
+      if title is None:
+        cap=""
+      else:
+        print("`title` is obsolete since `0.22.8`. Please use `caption`.")
+        cap=title
+    else:
+      cap=caption
+
+    self.sHtml += f'<div class="floating-box">{html}{self._getCaption(cap)}</div>'
+    return self
 
   def add_separator(self, size=3):
     """
     add a (poor) separation between elements in a row
     """
     self.add_html("&nbsp;" * size)
+    return self
 
-  def add_plot(self, oAxes, title=""):
+  def add_plot(self, oAxes, caption=None, title=None):
     """
     Add a PNG representation of a Matplotlib Axes object
+    (title is an obsolete parameter)
     """
+    if caption is None:
+      if title is None:
+        cap=""
+      else:
+        print("`title` is obsolete since `0.22.8`. Please use `caption`.")
+        cap=title
+    else:
+      cap=caption
+
     Bio = io.BytesIO()  # bytes buffer for the plot
     fig = oAxes.get_figure()
     fig.canvas.print_png(Bio)  # make a png of the plot in the buffer
 
     # encode the bytes as string using base 64
     sB64Img = base64.b64encode(Bio.getvalue()).decode()
-    self.sHtml += f'<div class="floating-box"><img src="data:image/png;base64,{sB64Img}\n">{self._getTitle(title)}</div>'
+    self.sHtml += f'<div class="floating-box"><img src="data:image/png;base64,{sB64Img}\n">{self._getCaption(cap)}</div>'
     plt.close()
+    return self
 
   def new_line(self):
     """
     add a breakline (a new row)
     """
     self.sHtml += '<br/>'
+    return self
 
   def html(self):
     """
@@ -142,26 +167,46 @@ class FlowLayout(object):
     IPython.display.display(self.html())
     self.clear()
 
+  def add(self,obj, caption=None, title=None):
+    """
+    add an element in the row by trying to treat it as plot or html if possible.
+    (title is an obsolete parameter)
+    """
+    if caption is None:
+      if title is None:
+        cap=""
+      else:
+        print("`title` is obsolete since `0.22.8`. Please use `caption`.")
+        cap=title
+    else:
+      cap=caption
+
+    if hasattr(obj, "get_figure"):
+      self.add_plot(obj, cap)
+    elif hasattr(obj,"_repr_html_"):
+      self.add_html(obj._repr_html_(),cap)
+    else:
+      self.add_html(obj,cap)
+
+    return self
+
   def row(self, *args, captions=None):
+    """
+    Create a row with flow with the same syntax as `pyAgrum.lib.notebook.sideBySide`.
+    """
     self.clear()
     for i, arg in enumerate(args):
       if captions is None:
-        t = ""
+        self.add(arg)
       else:
-        t = captions[i]
-
-      if hasattr(arg, "get_figure"):
-        self.add_plot(arg, title=t)
-      elif hasattr(arg, "_repr_html_"):
-        self.add_html(arg._repr_html_(), title=t)
-      else:
-        self.add_html(arg, title=t)
+        self.add(arg,captions[i])
 
     self.display()
 
 
 flow = FlowLayout()
 
+
 def configuration():
   """
   Display the collection of dependance and versions
@@ -174,60 +219,23 @@ def configuration():
   packages["OS"] = "%s [%s]" % (os.name, sys.platform)
   packages["Python"] = sys.version
   packages["IPython"] = IPython.__version__
-  packages["MatPlotLib"] = mpl.__version__
+  packages["Matplotlib"] = mpl.__version__
   packages["Numpy"] = np.__version__
+  packages["pyDot"] = dot.__version__
   packages["pyAgrum"] = gum.__version__
 
-  res = "<table width='100%'><tr><th>Library</th><th>Version</th></tr>"
+  res = "<table><tr><th>Library</th><th>Version</th></tr>"
 
   for name in packages:
     res += "<tr><td>%s</td><td>%s</td></tr>" % (name, packages[name])
 
   res += "</table><div align='right'><small>%s</small></div>" % time.strftime(
-    '%a %b %d %H:%M:%S %Y %Z')
+    '%a %b %d %H:%M:%S %Y %Z'
+  )
 
   IPython.display.display(IPython.display.HTML(res))
 
 
-def __insertLinkedSVGs(mainSvg):
-  re_buggwhitespace = re.compile(r"(<image [^>]*>)")
-  re_images = re.compile(r"(<image [^>]*>)")
-  re_xlink = re.compile(r"xlink:href=\"([^\"]*)")
-  re_viewbox = re.compile(r"(viewBox=\"[^\"]*\")")
-
-  # analyze mainSvg (find the secondary svgs)
-  __fragments = {}
-  for img in re.finditer(re_images, mainSvg):
-    # print(img)
-    secondarySvg = re.findall(re_xlink, img.group(1))[0]
-    content = ""
-    with open(secondarySvg, encoding='utf8') as f:
-      inSvg = False
-      for line in f:
-        if line[0:4] == "<svg":
-          inSvg = True
-          viewBox = re.findall(re_viewbox, line)[0]
-          # print("VIEWBOX {}".format(viewBox))
-        elif inSvg:
-          content += line
-    __fragments[secondarySvg] = (viewBox, content)
-
-  if len(__fragments) > 0:
-    # replace image tags by svg tags
-    img2svg = re.sub(r"<image ([^>]*)/>", "<svg \g<1>>", mainSvg)
-
-    # insert secondaries into main
-    def ___insertSecondarySvgs(matchObj):
-      vb, code = __fragments[matchObj.group(1)]
-      return vb + matchObj.group(2) + code
-
-    mainSvg = re.sub(r'xlink:href="([^"]*)"(.*>)',
-                     ___insertSecondarySvgs, img2svg)
-
-  # remove buggy white-space (for notebooks)
-  mainSvg = mainSvg.replace("white-space:pre;", "")
-  return mainSvg
-
 
 def _reprGraph(gr, size, asString, format=None):
   """
@@ -243,7 +251,7 @@ def _reprGraph(gr, size, asString, format=None):
     format = gum.config["notebook", "graph_format"]
 
   if format == "svg":
-    gsvg = IPython.display.SVG(__insertLinkedSVGs(gr.create_svg().decode('utf-8')))
+    gsvg = IPython.display.SVG(prepareLinksForSVG(gr.create_svg(encoding="utf-8").decode('utf-8')))
     if asString:
       return gsvg.data
     else:
@@ -251,7 +259,7 @@ def _reprGraph(gr, size, asString, format=None):
   else:
     i = IPython.core.display.Image(format="png", data=gr.create_png())
     if asString:
-      return '<img style="margin:0" src="data:image/png;base64,{}"/>'.format(encodebytes(i.data).decode())
+      return f'<img style="margin:0" src="data:image/png;base64,{encodebytes(i.data).decode()}"/>'
     else:
       IPython.core.display.display_png(i)
 
@@ -285,16 +293,21 @@ def getGraph(gr, size=None):
 
 
 def _from_dotstring(dotstring):
-  g = dot.graph_from_dot_data(dotstring)
+  g = dot.graph_from_dot_data(dotstring)[0]
+
+  # workaround for some badly parsed graph (pyparsing>=3.03)
+  g.del_node('"\\n"')
+  g.del_node('"\\n\\n"')
+
   g.set_bgcolor("transparent")
   for e in g.get_edges():
     if e.get_color() is None:
-      e.set_color(gum.getBlackInTheme())
+      e.set_color(getBlackInTheme())
   for n in g.get_nodes():
     if n.get_color() is None:
-      n.set_color(gum.getBlackInTheme())
+      n.set_color(getBlackInTheme())
     if n.get_fontcolor() is None:
-      n.set_fontcolor(gum.getBlackInTheme())
+      n.set_fontcolor(getBlackInTheme())
   return g
 
 
@@ -362,10 +375,14 @@ def showBNDiff(bn1, bn2, size=None):
   cmp = GraphicalBNComparator(bn1, bn2)
   showGraph(cmp.dotDiff(), size)
 
-def showInformation(*args,**kwargs):
-  print("[pyAgrum] pyAgrum.lib.notebook.showInformation is deprecated since 0.20.2. Please use pyAgrum.lib.explain.showInfomation instead.")
+
+def showInformation(*args, **kwargs):
+  print(
+    "[pyAgrum] pyAgrum.lib.notebook.showInformation is deprecated since 0.20.2. Please use pyAgrum.lib.explain.showInfomation instead."
+  )
   import pyAgrum.lib.explain as explain
-  explain.showInformation(*args,**kwargs)
+  explain.showInformation(*args, **kwargs)
+
 
 def showJunctionTree(bn, withNames=True, size=None):
   """
@@ -429,7 +446,7 @@ def showProba(p, scale=1.0):
 
 def _getMatplotFig(fig):
   bio = io.BytesIO()  # bytes buffer for the plot
-  fig.savefig(bio, format = 'png', bbox_inches='tight') #.canvas.print_png(bio)  # make a png of the plot in the buffer
+  fig.savefig(bio, format='png', bbox_inches='tight')  # .canvas.print_png(bio)  # make a png of the plot in the buffer
 
   # encode the bytes as string using base 64
   sB64Img = base64.b64encode(bio.getvalue()).decode()
@@ -512,15 +529,16 @@ def animApproximationScheme(apsc, scale=np.log10):
   f = plt.gcf()
 
   h = gum.PythonApproximationListener(
-    apsc._asIApproximationSchemeConfiguration())
+    apsc._asIApproximationSchemeConfiguration()
+  )
   apsc.setVerbosity(True)
   apsc.listener = h
 
   def stopper(x):
     IPython.display.clear_output(True)
     plt.title(
-      "{0} \n Time : {1} s | Iterations : {2} | Espilon : {3}".format(x, apsc.currentTime(), apsc.nbrIterations(),
-                                                                      apsc.epsilon()))
+      f"{x} \n Time : {apsc.currentTime()}s | Iterations : {apsc.nbrIterations()} | Epsilon : {apsc.epsilon()}"
+    )
 
   def progresser(x, y, z):
     if len(apsc.history()) < 10:
@@ -542,14 +560,14 @@ def showApproximationScheme(apsc, scale=np.log10):
     else:
       plt.xlim(1, len(apsc.history()))
     plt.title(
-      "Time : {} s | Iterations : {} | Espilon : {}".format(apsc.currentTime(),
-                                                            apsc.nbrIterations(),
-                                                            apsc.epsilon()))
+      f"Time : {apsc.currentTime()}s | Iterations : {apsc.nbrIterations()} | Epsilon : {apsc.epsilon()}"
+    )
     plt.plot(scale(apsc.history()), 'g')
 
 
 def showMN(mn, view=None, size=None, nodeColor=None, factorColor=None, edgeWidth=None, edgeColor=None, cmap=None,
-           cmapEdge=None):
+           cmapEdge=None
+           ):
   """
   show a Markov network
 
@@ -575,7 +593,8 @@ def showMN(mn, view=None, size=None, nodeColor=None, factorColor=None, edgeWidth
 
   if view == "graph":
     dottxt = MN2UGdot(mn, size, nodeColor, edgeWidth,
-                      edgeColor, cmap, cmapEdge)
+                      edgeColor, cmap, cmapEdge
+                      )
   else:
     dottxt = MN2FactorGraphdot(mn, size, nodeColor, factorColor, cmapNode=cmap)
 
@@ -655,7 +674,8 @@ def showCN(cn, size=None, nodeColor=None, arcWidth=None, arcColor=None, cmap=Non
 
 
 def getMN(mn, view=None, size=None, nodeColor=None, factorColor=None, edgeWidth=None, edgeColor=None, cmap=None,
-          cmapEdge=None):
+          cmapEdge=None
+          ):
   """
   get an HTML string for a Markov network
 
@@ -681,7 +701,8 @@ def getMN(mn, view=None, size=None, nodeColor=None, factorColor=None, edgeWidth=
 
   if view == "graph":
     dottxt = MN2UGdot(mn, size, nodeColor, edgeWidth,
-                      edgeColor, cmap, cmapEdge)
+                      edgeColor, cmap, cmapEdge
+                      )
   else:
     dottxt = MN2FactorGraphdot(mn, size, nodeColor, factorColor, cmapNode=cmap)
 
@@ -733,53 +754,6 @@ def getCN(cn, size=None, nodeColor=None, arcWidth=None, arcColor=None, cmap=None
 
   return getGraph(CN2dot(cn, size, nodeColor, arcWidth, arcColor, cmap, cmapArc), size)
 
-def _get_showInference(model, engine=None, evs=None, targets=None, size=None,
-                       nodeColor=None, factorColor=None, arcWidth=None,
-                       arcColor=None, cmap=None, cmapArc=None, graph=None, view=None):
-  if size is None:
-    size = gum.config["notebook", "default_graph_inference_size"]
-
-  if evs is None:
-    evs = {}
-
-  if targets is None:
-    targets = {}
-
-  if isinstance(model, gum.BayesNet):
-    if engine is None:
-      engine = gum.LazyPropagation(model)
-    return BNinference2dot(model, size=size, engine=engine, evs=evs, targets=targets, nodeColor=nodeColor,
-                           arcWidth=arcWidth,
-                           arcColor=arcColor,
-                           cmapNode=cmap, cmapArc=cmapArc)
-  elif isinstance(model, gum.MarkovNet):
-    if view is None:
-      view = gum.config["notebook", "default_markovnetwork_view"]
-      if engine is None:
-        engine = gum.ShaferShenoyMNInference(model)
-
-      if view == "graph":
-        return MNinference2UGdot(model, size=size, engine=engine, evs=evs, targets=targets, nodeColor=nodeColor,
-                                 factorColor=factorColor,
-                                 arcWidth=arcWidth, arcColor=arcColor, cmapNode=cmap, cmapArc=cmapArc)
-      else:
-        return MNinference2FactorGraphdot(model, size=size, engine=engine, evs=evs, targets=targets,
-                                          nodeColor=nodeColor,
-                                          factorColor=factorColor, cmapNode=cmap)
-  elif isinstance(model, gum.InfluenceDiagram):
-    if engine is None:
-      engine = gum.ShaferShenoyLIMIDInference(model)
-    return LIMIDinference2dot(model, size=size, engine=engine, evs=evs, targets=targets)
-  elif isinstance(model, gum.CredalNet):
-    if engine is None:
-      engine = gum.CNMonteCarloSampling(model)
-    return CNinference2dot(model, size=size, engine=engine, evs=evs, targets=targets, nodeColor=nodeColor,
-                           arcWidth=arcWidth, arcColor=arcColor, cmapNode=cmap)
-  else:
-    raise gum.InvalidArgument(
-      "Argument model should be a PGM (BayesNet, MarkovNet or Influence Diagram")
-
-
 def showInference(model, **kwargs):
   """
   show pydot graph for an inference in a notebook
@@ -804,7 +778,7 @@ def showInference(model, **kwargs):
   else:
     size = gum.config["notebook", "default_graph_inference_size"]
 
-  showGraph(_get_showInference(model, **kwargs), size)
+  showGraph(prepareShowInference(model, **kwargs), size)
 
 
 def getInference(model, **kwargs):
@@ -834,7 +808,7 @@ def getInference(model, **kwargs):
   else:
     size = gum.config["notebook", "default_graph_inference_size"]
 
-  grinf = _get_showInference(model, **kwargs)
+  grinf = prepareShowInference(model, **kwargs)
   return getGraph(grinf, size)
 
 
@@ -853,18 +827,9 @@ def _reprPotential(pot, digits=None, withColors=True, varnames=None, asString=Fa
   """
   from IPython.core.display import HTML
 
-  def hex_to_rgb(v):
-    """
-    from #FFFFFF to 255,255,255
-    :param value: the rbg string
-    :return: the triple
-    """
-    value = v.lstrip('#')
-    lv = len(value)
-    return tuple(int(value[i:i + lv // 3], 16) for i in range(0, lv, lv // 3))
 
-  r0,g0,b0=hex_to_rgb(gum.config['notebook', 'potential_color_0'])
-  r1,g1,b1=hex_to_rgb(gum.config['notebook', 'potential_color_1'])
+  r0, g0, b0 = gumcols.hex2rgb(gum.config['notebook', 'potential_color_0'])
+  r1, g1, b1 = gumcols.hex2rgb(gum.config['notebook', 'potential_color_1'])
 
   def _rgb(r, g, b):
     return '#%02x%02x%02x' % (r, g, b)
@@ -872,14 +837,13 @@ def _reprPotential(pot, digits=None, withColors=True, varnames=None, asString=Fa
   def _mkCell(val):
     s = "<td style='"
     if withColors and (0 <= val <= 1):
-      r = int(r0 + val * (r1-r0))
-      g = int(g0 + val * (g1-g0))
-      b = int(b0 + val * (b1-b0))
+      r = int(r0 + val * (r1 - r0))
+      g = int(g0 + val * (g1 - g0))
+      b = int(b0 + val * (b1 - b0))
 
-      brightness = r * 0.299 + g * 0.587 + b * 0.114
-      tx = "white" if brightness <=153 else "black"
+      tx = gumcols.rgb2brightness(r,g,b)
 
-      s += "color:"+tx+";background-color:" + _rgb(r, g, b) + ";"
+      s += "color:" + tx + ";background-color:" + _rgb(r, g, b) + ";"
     s += "text-align:right;'>{:." + str(digits) + "f}</td>"
     return s.format(val)
 
@@ -893,12 +857,14 @@ def _reprPotential(pot, digits=None, withColors=True, varnames=None, asString=Fa
   html.append('<table style="border:1px solid black;">')
   if pot.empty():
     html.append(
-      "<tr><th>&nbsp;</th></tr>")
+      "<tr><th>&nbsp;</th></tr>"
+    )
     html.append("<tr>" + _mkCell(pot.get(gum.Instantiation())) + "</tr>")
   else:
     if varnames is not None and len(varnames) != pot.nbrDim():
       raise ValueError(
-        "varnames contains {} values instead of the needed {} values.".format(len(varnames), pot.nbrDim()))
+        f"varnames contains {len(varnames)} value(s) instead of the needed {pot.nbrDim()} value(s)."
+      )
 
     nparents = pot.nbrDim() - 1
     var = pot.variable(0)
@@ -906,15 +872,15 @@ def _reprPotential(pot, digits=None, withColors=True, varnames=None, asString=Fa
 
     # first line
     if nparents > 0:
-      s = """<tr><th colspan='{}'></th>
-      <th colspan='{}' style='border:1px solid black;color:black;background-color:#808080;'><center>{}</center>
-      </th></tr>""".format(
-        nparents, var.domainSize(), varname)
-      html.append(s)
+      html.append(f"""<tr><th colspan='{nparents}'></th>
+      <th colspan='{var.domainSize()}' style='border:1px solid black;color:black;background-color:#808080;'><center>{varname}</center>
+      </th></tr>"""
+                  )
     else:
-      s = """<tr style='border:1px solid black;color:black;background-color:#808080'>
-      <th colspan='{}'><center>{}</center></th></tr>""".format(var.domainSize(), varname)
-      html.append(s)
+      html.append(f"""<tr style='border:1px solid black;color:black;background-color:#808080'>
+      <th colspan='{var.domainSize()}'><center>{varname}</center></th></tr>"""
+                  )
+
     # second line
     s = "<tr>"
     if nparents > 0:
@@ -925,11 +891,11 @@ def _reprPotential(pot, digits=None, withColors=True, varnames=None, asString=Fa
         pmin, pmax, pinc = 0, nparents, 1
       for par in range(pmin, pmax, pinc):
         parent = pot.var_names[par] if varnames is None else varnames[par]
-        s += "<th style='border:1px solid black;color:black;background-color:#808080'><center>{}</center></th>".format(
-          parent)
+        s += f"<th style='border:1px solid black;color:black;background-color:#808080'><center>{parent}</center></th>"
+
     for label in var.labels():
-      s += """<th style='border:1px solid black;border-bottom-style: double;color:black;background-color:#BBBBBB'>
-      <center>{}</center></th>""".format(label)
+      s += f"""<th style='border:1px solid black;border-bottom-style: double;color:black;background-color:#BBBBBB'>
+      <center>{label}</center></th>"""
     s += '</tr>'
 
     html.append(s)
@@ -952,12 +918,11 @@ def _reprPotential(pot, digits=None, withColors=True, varnames=None, asString=Fa
       for par in range(pmin, pmax, pinc):
         label = inst.variable(par).label(inst.val(par))
         if par == 1 or gum.config["notebook", "potential_parent_values"] == "nomerge":
-          s += "<th style='border:1px solid black;color:black;background-color:#BBBBBB'><center>{}</center></th>".format(
-            label)
+          s += f"<th style='border:1px solid black;color:black;background-color:#BBBBBB'><center>{label}</center></th>"
         else:
           if sum([inst.val(i) for i in range(1, par)]) == 0:
-            s += """<th style='border:1px solid black;color:black;background-color:#BBBBBB;' rowspan = '{}'>
-            <center>{}</center></th>""".format(offset[par], label)
+            s += f"""<th style='border:1px solid black;color:black;background-color:#BBBBBB;' rowspan = '{offset[par]}'>
+            <center>{label}</center></th>"""
       for j in range(pot.variable(0).domainSize()):
         s += _mkCell(pot.get(inst))
         inst.inc()
@@ -1041,31 +1006,42 @@ def getPotential(pot, digits=None, withColors=None, varnames=None):
 
   return _reprPotential(pot, digits, withColors, varnames, asString=True)
 
+def showCPTs(bn):
+  flow.clear()
+  for i in bn.names():
+    flow.add_html(getPotential(bn.cpt(i)))
+  flow.display()
+
 
 def getSideBySide(*args, **kwargs):
   """
   create an HTML table for args as string (using string, _repr_html_() or str())
 
   :param args: HMTL fragments as string arg, arg._repr_html_() or str(arg)
-  :param captions: list of strings (captions)
+  :param captions: list of strings (optional)
+  :param valign: vertical position in the row (top|middle|bottom, middle by default)
+  :param ncols: number of columns (infinite by default)
   :return: a string representing the table
   """
-  vals = {'captions', 'valign'}
+  vals = {'captions', 'valign','ncols'}
   if not set(kwargs.keys()).issubset(vals):
-    raise TypeError("sideBySide() got unexpected keyword argument(s) : '{}'".format(
-      set(kwargs.keys()).difference(vals)))
+    raise TypeError(f"sideBySide() got unexpected keyword argument(s) : '{set(kwargs.keys()).difference(vals)}'")
 
   if 'captions' in kwargs:
     captions = kwargs['captions']
   else:
     captions = None
 
-  if 'valign' in kwargs:
-    v_align = 'vertical-align:' + kwargs['valign'] + ';'
+  if 'valign' in kwargs and kwargs['valign'] in ['top','middle','bottom'] :
+    v_align = f"vertical-align:{kwargs['valign']};"
   else:
-    v_align = ""
+    v_align = f"vertical-align:middle;"
 
-  s = '<table style="border-style: hidden; border-collapse: collapse;" width="100%">'
+  ncols=None
+  if 'ncols' in kwargs:
+    ncols=int(kwargs['ncols'])
+    if ncols<1:
+      ncols=1
 
   def reprHTML(s):
     if isinstance(s, str):
@@ -1075,22 +1051,16 @@ def getSideBySide(*args, **kwargs):
     else:
       return str(s)
 
-  s += '<tr><td style="border-top:hidden;border-bottom:hidden;' + v_align + '"><div align="center" style="' + v_align \
-       + '">'
-  s += (
-     '</div></td><td style="border-top:hidden;border-bottom:hidden;' + v_align + '"><div align="center" style="' +
-     v_align + '">').join(
-    [reprHTML(arg)
-     for arg in args])
-  s += '</div></td></tr>'
-
-  if captions is not None:
-    s += '<tr><td style="border-top:hidden;border-bottom:hidden;"><div align="center"><small>'
-    s += '</small></div></td><td style="border-top:hidden;border-bottom:hidden;"><div align="center"><small>'.join(
-      captions)
-    s += '</small></div></td></tr>'
-
-  s += '</table>'
+  s = '<table style="border-style: hidden; border-collapse: collapse;" width="100%"><tr>'
+  for i in range(len(args)):
+    s += f'<td style="border-top:hidden;border-bottom:hidden;{v_align}"><div align="center" style="{v_align}">'
+    s+=reprHTML(args[i])
+    if captions is not None:
+      s+=f'<br><small><i>{captions[i]}</i></small>'
+    s+='</div></td>'
+    if ncols is not None and (i+1)%ncols==0:
+      s+='</tr><tr>'
+  s += '</tr></table>'
   return s
 
 
@@ -1109,19 +1079,21 @@ def getInferenceEngine(ie, inferenceCaption):
   display an inference as a BN+ lists of hard/soft evidence and list of targets
 
   :param gum.InferenceEngine ie: inference engine
-  :param string inferenceCaption: title for caption
+  :param string inferenceCaption: caption for the inference
 
   """
   t = '<div align="left"><ul>'
   if ie.nbrHardEvidence() > 0:
     t += "<li><b>hard evidence</b><br/>"
     t += ", ".join([ie.BN().variable(n).name()
-                    for n in ie.hardEvidenceNodes()])
+                    for n in ie.hardEvidenceNodes()]
+                   )
     t += "</li>"
   if ie.nbrSoftEvidence() > 0:
     t += "<li><b>soft evidence</b><br/>"
     t += ", ".join([ie.BN().variable(n).name()
-                    for n in ie.softEvidenceNodes()])
+                    for n in ie.softEvidenceNodes()]
+                   )
     t += "</li>"
   if ie.nbrTargets() > 0:
     t += "<li><b>target(s)</b><br/>"
@@ -1136,8 +1108,10 @@ def getInferenceEngine(ie, inferenceCaption):
       t += "<li><b>Joint target(s)</b><br/>"
       t += ", ".join(['['
                       + (", ".join([ie.BN().variable(n).name()
-                                    for n in ns]))
-                      + ']' for ns in ie.jointTargets()])
+                                    for n in ns]
+                                   ))
+                      + ']' for ns in ie.jointTargets()]
+                     )
       t += "</li>"
   t += '</ul></div>'
   return getSideBySide(getBN(ie.BN()), t, captions=[inferenceCaption, "Evidence and targets"])
@@ -1146,36 +1120,41 @@ def getInferenceEngine(ie, inferenceCaption):
 def getJT(jt, size=None):
   if gum.config["notebook", "junctiontree_with_names"] == "True":
     def cliqlabels(c):
-      return " ".join(
-        sorted([model.variable(n).name() for n in jt.clique(c)]))
+      labels=",".join(
+        sorted([model.variable(n).name() for n in jt.clique(c)])
+      )
+      return f"({c}):{labels}"
 
-    def cliqnames(
-       c):
+    def cliqnames(c):
       return "-".join(sorted([model.variable(n).name() for n in jt.clique(c)]))
 
     def seplabels(c1, c2):
-      return " ".join(
-        sorted([model.variable(n).name() for n in jt.separator(c1, c2)]))
+      return ",".join(
+        sorted([model.variable(n).name() for n in jt.separator(c1, c2)])
+      )
 
     def sepnames(c1, c2):
       return cliqnames(c1) + '+' + cliqnames(c2)
   else:
     def cliqlabels(c):
-      return " ".join([str(n) for n in sorted(jt.clique(c))])
+      ids=",".join([str(n) for n in sorted(jt.clique(c))])
+      return f"({c}):{ids}"
 
     def cliqnames(c):
       return "-".join([str(n) for n in sorted(jt.clique(c))])
 
     def seplabels(c1, c2):
-      return " ".join(
-        [str(n) for n in sorted(jt.separator(c1, c2))])
+      return ",".join(
+        [str(n) for n in sorted(jt.separator(c1, c2))]
+      )
 
     def sepnames(c1, c2):
       return cliqnames(c1) + '^' + cliqnames(c2)
 
   model = jt._engine._model
   name = model.propertyWithDefault(
-    "name", str(type(model)).split(".")[-1][:-2])
+    "name", str(type(model)).split(".")[-1][:-2]
+  )
   graph = dot.Dot(graph_type='graph', graph_name=name, bgcolor="transparent")
   for c in jt.nodes():
     graph.add_node(dot.Node('"' + cliqnames(c) + '"',
@@ -1185,7 +1164,9 @@ def getJT(jt, size=None):
                                                  "junctiontree_clique_bgcolor"],
                             fontcolor=gum.config["notebook",
                                                  "junctiontree_clique_fgcolor"],
-                            fontsize=gum.config["notebook", "junctiontree_clique_fontsize"]))
+                            fontsize=gum.config["notebook", "junctiontree_clique_fontsize"]
+                            )
+                   )
   for c1, c2 in jt.edges():
     graph.add_node(dot.Node('"' + sepnames(c1, c2) + '"',
                             label='"' + seplabels(c1, c2) + '"',
@@ -1195,12 +1176,18 @@ def getJT(jt, size=None):
                                                  "junctiontree_separator_bgcolor"],
                             fontcolor=gum.config["notebook",
                                                  "junctiontree_separator_fgcolor"],
-                            fontsize=gum.config["notebook", "junctiontree_separator_fontsize"]))
+                            fontsize=gum.config["notebook", "junctiontree_separator_fontsize"]
+                            )
+                   )
   for c1, c2 in jt.edges():
     graph.add_edge(dot.Edge('"' + cliqnames(c1) +
-                            '"', '"' + sepnames(c1, c2) + '"'))
+                            '"', '"' + sepnames(c1, c2) + '"'
+                            )
+                   )
     graph.add_edge(dot.Edge('"' + sepnames(c1, c2) +
-                            '"', '"' + cliqnames(c2) + '"'))
+                            '"', '"' + cliqnames(c2) + '"'
+                            )
+                   )
 
   if size is None:
     size = gum.config["notebook", "default_graph_size"]
@@ -1242,87 +1229,16 @@ def show(model, **kwargs):
   elif isinstance(model, gum.Potential):
     showPotential(model)
   elif hasattr(model, "toDot"):
-    showDot(dot.graph_from_dot_data(model.toDot()), **kwargs)
+    g=dot.graph_from_dot_data(model.toDot())[0]
+
+    # workaround for some badly parsed graph (pyparsing>=3.03)
+    g.del_node('"\\n"')
+
+    showDot(g, **kwargs)
   else:
     raise gum.InvalidArgument(
-      "Argument model should be a PGM (BayesNet, MarkovNet, Influence Diagram or Potential or ...")
-
-
-def export(model, filename, **kwargs):
-  """
-  export the graphical representation of the model in filename (png, pdf,etc.)
-
-  :param GraphicalModel model: the model to show (pyAgrum.BayesNet, pyAgrum.MarkovNet, pyAgrum.InfluenceDiagram or pyAgrum.Potential)
-  :param str filename: the name of the resulting file (suffix in ['pdf', 'png', 'fig', 'jpg', 'svg', 'ps'])
-  """
-  format = filename.split(".")[-1]
-  if format not in ['pdf', 'png', 'fig', 'jpg', 'svg', 'ps']:
-    raise Exception(
-      f"{filename} in not a correct filename for export : extension '{format}' not in [pdf,png,fig,jpg,svg].")
-
-  if isinstance(model, gum.BayesNet):
-    fig = BN2dot(model, **kwargs)
-  elif isinstance(model, gum.MarkovNet):
-    if gum.config["notebook", "default_markovnetwork_view"] == "graph":
-      fig = MN2UGdot(model, **kwargs)
-    else:
-      fig = MN2FactorGraphdot(model, **kwargs)
-  elif isinstance(model, gum.InfluenceDiagram):
-    fig = ID2dot(model, **kwargs)
-  elif isinstance(model, gum.CredalNet):
-    fig = CN2dot(model, **kwargs)
-  elif hasattr(model, "toDot"):
-    fig = dot.graph_from_dot_data(model.toDot(), **kwargs)
-  else:
-    raise gum.InvalidArgument(
-      "Argument model should be a PGM (BayesNet, MarkovNet or Influence Diagram")
-  fig.write(filename, format=format)
-
-
-def exportInference(model, filename, **kwargs):
-  """
-  the graphical representation of an inference in a notebook
-
-  :param GraphicalModel model: the model in which to infer (pyAgrum.BayesNet, pyAgrum.MarkovNet or
-          pyAgrum.InfluenceDiagram)
-  :param str filename: the name of the resulting file (suffix in ['pdf', 'png', 'ps'])
-  :param gum.Inference engine: inference algorithm used. If None, gum.LazyPropagation will be used for BayesNet,
-          gum.ShaferShenoy for gum.MarkovNet and gum.ShaferShenoyLIMIDInference for gum.InfluenceDiagram.
-  :param dictionnary evs: map of evidence
-  :param set targets: set of targets
-  :param string size: size of the rendered graph
-  :param nodeColor: a nodeMap of values (between 0 and 1) to be shown as color of nodes (with special colors for 0 and 1)
-  :param factorColor: a nodeMap of values (between 0 and 1) to be shown as color of factors (in MarkovNet representation)
-  :param arcWidth: a arcMap of values to be shown as width of arcs
-  :param arcColor: a arcMap of values (between 0 and 1) to be shown as color of arcs
-  :param cmap: color map to show the color of nodes and arcs
-  :param cmapArc: color map to show the vals of Arcs.
-  :param graph: only shows nodes that have their id in the graph (and not in the whole BN)
-  :param view: graph | factorgraph | None (default) for Markov network
-  :return: the desired representation of the inference
-  """
-  format = filename.split(".")[-1]
-  if format not in ['pdf', 'png', 'ps']:
-    raise Exception(
-      f"{filename} in not a correct filename for export : extension '{format}' not in [pdf,png,ps].")
-
-  import cairosvg
-
-  if "size" in kwargs:
-    size = kwargs['size']
-  else:
-    size = gum.config["notebook", "default_graph_inference_size"]
-
-  svgtxt = _reprGraph(
-    _get_showInference(model, **kwargs), size=size, asString=True, format="svg")
-
-  if format == "pdf":
-    cairosvg.svg2pdf(bytestring=svgtxt, write_to=filename)
-  elif format == "png":
-    cairosvg.svg2png(bytestring=svgtxt, write_to=filename)
-  else:  # format=="ps"
-    cairosvg.svg2ps(bytestring=svgtxt, write_to=filename)
-
+      "Argument model should be a PGM (BayesNet, MarkovNet, Influence Diagram or Potential or ..."
+    )
 
 def _update_config():
   # hook to control some parameters for notebook when config changes
@@ -1338,7 +1254,8 @@ except NameError as e:
 
   warnings.warn("""
   ** pyAgrum.lib.notebook has to be imported from an IPython's instance (mainly notebook).
-  """)
+  """
+                )
 else:
   gum.config.add_hook(_update_config)
   gum.config.run_hooks()
@@ -1355,7 +1272,8 @@ else:
 
   gum.Potential._repr_html_ = lambda self: getPotential(self)
   gum.LazyPropagation._repr_html_ = lambda self: getInferenceEngine(
-    self, "Lazy Propagation on this BN")
+    self, "Lazy Propagation on this BN"
+  )
 
   gum.UndiGraph._repr_html_ = lambda self: getDot(self.toDot())
   gum.DiGraph._repr_html_ = lambda self: getDot(self.toDot())
@@ -1363,3 +1281,5 @@ else:
   gum.DAG._repr_html_ = lambda self: getDot(self.toDot())
   gum.EssentialGraph._repr_html_ = lambda self: getDot(self.toDot())
   gum.MarkovBlanket._repr_html_ = lambda self: getDot(self.toDot())
+
+  dot.Dot._repr_html_ = lambda self: getGraph(self)
